# -*- coding: utf-8 -*-

import json
from flask import Flask, request, jsonify, make_response, render_template
from playhouse.shortcuts import model_to_dict
from rssbot.models import Feed, Item

app = Flask(__name__, static_url_path='/static', static_folder='static')


@app.route('/api/feeds', methods=['GET', 'POST', 'DELETE'])
def feeds():
    if request.method == 'GET':
        all_feeds = Feed.select()
        result = list()
        for feed in all_feeds:
            temp = model_to_dict(feed)
            temp['create_time'] = temp['create_time'].strftime('%Y-%m-%d %H:%M')
            temp['last_check'] = temp['last_check'].strftime('%Y-%m-%d %H:%M')
            temp['last_add'] = temp['last_add'].strftime('%Y-%m-%d %H:%M')
            temp['keywords'] = json.loads(temp['keywords']) if temp['keywords'] else []
            result.append(temp)
        return jsonify(result)

    if request.method == 'POST':
        posted_feed = request.get_json()
        if 'id' in posted_feed.keys():
            query = Feed.update(**posted_feed).where(Feed.id == posted_feed['id'])
            query.execute()
        else:
            Feed.create(**posted_feed)
        return make_response('OK', 200)

    if request.method == 'DELETE':
        posted_feed = request.get_json()
        if 'id' in posted_feed.keys():
            query = Feed.delete().where(Feed.id == posted_feed['id'])
            query.execute()
            return make_response('OK', 200)


@app.route('/api/items', methods=['GET'])
def items():
    all_items = Item.select()
    result = list()
    for item in all_items:
        temp = model_to_dict(item)
        temp['seen_time'] = temp['seen_time'].strftime('%Y-%m-%d %H:%M')
        temp['publish_time'] = temp['publish_time'].strftime('%Y-%m-%d %H:%M')
        result.append(temp)
    return jsonify(result)


@app.route('/')
def index():
    return render_template('index.html')



# -*- coding:utf-8 -*-

import six
import sys
import time
import logging

try:
    import curses

    assert curses
except ImportError:
    curses = None

logger = logging.getLogger("rss-transmission")


def enable_pretty_logging(logger, level='info'):
    """
    按照配置开启 log 的格式化优化。
    :param logger: 配置的 logger 对象
    :param level: 要为 logger 设置的等级
    """
    logger.setLevel(getattr(logging, level.upper()))

    if not logger.handlers:
        # Set up color if we are in a tty and curses is installed
        color = False
        if curses and sys.stderr.isatty():
            try:
                curses.setupterm()
                if curses.tigetnum("colors") > 0:
                    color = True
            finally:
                pass
        channel = logging.StreamHandler()
        channel.setFormatter(_LogFormatter(color=color))
        logger.addHandler(channel)


class _LogFormatter(logging.Formatter):
    def __init__(self, color, *args, **kwargs):
        logging.Formatter.__init__(self, *args, **kwargs)
        self._color = color
        if color:
            fg_color = (curses.tigetstr("setaf") or
                        curses.tigetstr("setf") or "")
            if (3, 0) < sys.version_info < (3, 2, 3):
                fg_color = six.text_type(fg_color, "ascii")
            self._colors = {
                logging.DEBUG: six.text_type(
                    curses.tparm(fg_color, 4),
                    "ascii"
                ),  # Blue
                logging.INFO: six.text_type(
                    curses.tparm(fg_color, 2),
                    "ascii"
                ),  # Green
                logging.WARNING: six.text_type(
                    curses.tparm(fg_color, 3),
                    "ascii"
                ),  # Yellow
                logging.ERROR: six.text_type(
                    curses.tparm(fg_color, 1),
                    "ascii"
                ),  # Red
            }
            self._normal = six.text_type(curses.tigetstr("sgr0"), "ascii")

    def format(self, record):
        try:
            record.message = record.getMessage()
        except Exception as e:
            record.message = "Bad message (%r): %r" % (e, record.__dict__)
        record.asctime = time.strftime(
            "%y%m%d %H:%M:%S", self.converter(record.created))
        prefix = '[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d]' % \
                 record.__dict__
        if self._color:
            prefix = (self._colors.get(record.levelno, self._normal) +
                      prefix + self._normal)
        formatted = prefix + " " + record.message
        if record.exc_info:
            if not record.exc_text:
                record.exc_text = self.formatException(record.exc_info)
        if record.exc_text:
            formatted = formatted.rstrip() + "\n" + record.exc_text
        return formatted.replace("\n", "\n    ")



# -*- coding: utf-8 -*-

import os
import json


class Config(dict):
    def __init__(self):
        super(Config, self).__init__()
        self.config_update(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'default_settings.json'))

    def config_update(self, config_path=None):
        if not config_path:
            config_path = os.path.join(self['work_dir'], 'settings.json')
        if not os.path.exists(config_path):
            raise RuntimeError('Config path "{}" does not exist!'.format(config_path))
        with open(config_path, 'r') as f:
            config = json.loads(f.read())
        for k in config.keys():
            self[k] = config[k]



# -*- coding: utf-8 -*-

import feedparser
import requests
import datetime
import json
from rssbot.models import Feed, Item
from rssbot.logger import logger


class Feeder(object):
    def add(self, url, save_path):
        Feed.create(url=url, save_path=save_path)
        return True

    def get_all_feeds(self):
        feeds = Feed.select()
        return feeds

    def parse_items(self, content):
        d = feedparser.parse(content)
        result = list()
        for entry in d['entries']:
            item = dict(
                title=entry['title'],
                publish_time=datetime.datetime(*tuple(entry['published_parsed'])[:-2])
            )
            for link in entry['links']:
                if link['type'] == 'application/x-bittorrent':
                    item['magnet_link'] = link['href']
                    break
            result.append(item)
        return result

    def update(self):
        feeds = self.get_all_feeds()
        result = list()
        for feed in feeds:
            logger.debug('Updating for feed: {}'.format(feed.title))
            r = requests.get(feed.url)
            items = self.parse_items(r.text)
            for item in items:
                logger.debug('\tGet item: {}'.format(item['title']))
                query = Item.select().where(Item.title == item['title'])
                if not query.exists():
                    try:
                        logger.debug('\tItem does not exist.')
                        keywords = json.loads(feed.keywords)
                    except:
                        keywords = []
                    new_item = Item(**item, feed=feed)
                    new_item.save()
                    logger.debug('\tSaving item done.')
                    if len(keywords) > 0:
                        logger.debug('\tChecking keywords.')
                        for keyword in keywords:
                            if keyword in item['title']:
                                logger.debug('\tKeyword in title.')
                                result.append(new_item)
                                Feed.update(last_add=datetime.datetime.now()).where(Feed.id == feed.id).execute()
                                break
                    else:
                        logger.debug('\tNo keywords.Skip checking.')
                        result.append(new_item)
                        Feed.update(last_add=datetime.datetime.now()).where(Feed.id == feed.id).execute()
                else:
                    logger.debug('\tItem exists.')
            logger.debug('Updating feed last check.')
            Feed.update(last_check=datetime.datetime.now()).where(Feed.id == feed.id).execute()
            logger.debug('Updating last check done.')
        return result



# -*- coding: utf-8 -*-

import datetime
from peewee import Model, CharField, ForeignKeyField
from playhouse.apsw_ext import APSWDatabase, DateTimeField

db = APSWDatabase(None)


class BaseModel(Model):
    class Meta:
        database = db


class Feed(BaseModel):
    url = CharField()
    save_path = CharField()
    title = CharField(default='No name')
    create_time = DateTimeField(default=datetime.datetime.now)
    keywords = CharField(default='[]')
    last_check = DateTimeField(default=datetime.datetime.now)
    last_add = DateTimeField(default=datetime.datetime.now)


class Item(BaseModel):
    title = CharField()
    magnet_link = CharField()
    feed = ForeignKeyField(Feed)
    seen_time = DateTimeField(default=datetime.datetime.now)
    publish_time = DateTimeField()



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from rssbot import __version__

setup(
    name='rss-transmission',
    version=__version__,
    description='Rss plugin for transmission with a web ui',
    long_description=open("README.md").read(),
    keywords='rss transmission webui',
    author="helloqiu",
    author_email="helloqiu95@gmail.com",
    url='https://github.com/helloqiu/rss-transmission',
    packages=find_packages(),
    include_package_data=True,
    install_requires=open("requirements.txt").readlines(),
    license="GPLv3",
    classifiers=[
        "Development Status :: 1 - Planning",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: MacOS",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python"
    ],
    setup_requires=[
        "pytest-runner",
    ],
    tests_require=[
        "pytest",
    ],
)



# -*- coding: utf-8 -*-


import os
import time
from threading import Thread
import transmissionrpc
from rssbot.config import Config
from rssbot.feeds import Feeder
from rssbot.logger import logger, enable_pretty_logging
from rssbot.models import db, Feed, Item
from rssbot.web import app
from gevent.wsgi import WSGIServer


class Worker(object):
    def __init__(self, config=None):
        self.logger = logger
        enable_pretty_logging(self.logger)
        self.logger.info('Rss transmission start.')
        if config is None:
            self.config = Config()
        else:
            self.config = config
        if not os.path.exists(self.config['work_dir']):
            self.logger.info('Default work dir is not exist.Try to create one.')
            os.makedirs(self.config['work_dir'])
        db.init(os.path.join(self.config['work_dir'], 'rss-transmission.db'))
        Feed.create_table(True)
        Item.create_table(True)
        self.feeder = Feeder()
        self.client = transmissionrpc.Client(
            'http://{}:{}{}'.format(
                self.config['server']['host'],
                self.config['server']['port'],
                self.config['server']['rpc_path']
            )
        )

    def work(self):
        while True:
            try:
                self.logger.debug('Start to update.')
                result = self.feeder.update()
                for item in result:
                    self.logger.info('Get new item:\nTitle: {}\nMagnet: {}'.format(item.title, item.magnet_link))
                    self.logger.debug('\tStart to add torrent to transmission.')
                    self.client.add_torrent(item.magnet_link, download_dir=item.feed.save_path)
                    self.logger.debug('\tAdding done.')
                if len(result) == 0:
                    self.logger.info('Nothing to do.')
                self.logger.info('Update is done.')
                time.sleep(self.config['update_interval'])
            except Exception as e:
                self.logger.info(e)

    def run(self):
        self.logger.info('Rss transmission is running.')
        server = WSGIServer(('0.0.0.0', 9092), app)
        t = Thread(target=self.work)
        t.start()
        server.serve_forever()
